@@ -3,7 +3,7 @@ function(y, n, q, xmu.1, p.xmu, xsum.1, p.xsum,
                      zdummy, qz,nz0, m, rid, EUID, nEU,
                      prior1, prior2, prior.beta, prior.Sigma, 
                      prec.int, prec.DN, lambda.L1, lambda.L2,lambda.ARD,
-                     scale.unif, scale.halft, link, n.chain, inits) 
+                     scale.unif, scale.halft, link, n.chain, inits, seed) 
 { 
   dataIn <- vector("list",21)
   names(dataIn) <- c("y","n","q","xmu.1","p.xmu","xsum.1","p.xsum",
@@ -32,8 +32,41 @@ function(y, n, q, xmu.1, p.xmu, xsum.1, p.xsum,
   if(grepl("unif", prior.Sigma)) dataIn[[21]] <- scale.unif
   if(grepl("halfcauchy",prior.Sigma)) dataIn[[21]] <- scale.halft             
   
-  
-  init <- function( ){
+  if(is.null(seed)){
+    init <- function( rngname, rngseed){
+      rho1 <- runif(1,-0.5,0.5)
+      rho2 <- runif(1,-0.5,0.5) 
+      rho3 <- runif(1, rho1*rho2 - sqrt((1-rho1^2)*(1-rho2^2)), 
+                    rho1*rho2 + sqrt((1-rho1^2)*(1-rho2^2)))
+      return(
+        list("tmp1" = rnorm(q,0,0.1),
+             "tmp2" = rnorm(q,0,0.1),
+             
+             "b.tmp" = array(rnorm((p.xmu-1)*4*q,0,0.1), c((p.xmu-1),q,4)),
+             "d.tmp" = array(rnorm((p.xsum-1)*4*q,0,0.1),c((p.xsum-1),q,4)),
+             
+             "sigmab.L1" =  matrix(runif((p.xmu-1)*q,0,2), (p.xmu-1),q), 
+             "sigmad.L1" =  matrix(runif((p.xsum-1)*q,0,2),(p.xsum-1),q),  
+             
+             "taub.ARD" =  matrix(runif((p.xmu-1)*q,0,2), (p.xmu-1),q), 
+             "taud.ARD" =  matrix(runif((p.xsum-1)*q,0,2),(p.xsum-1),q), 
+             
+             "taub.L2" =  runif(q,0,2), 
+             "taud.L2" =  runif(q,0,2),
+             
+             "sigma.VC1" = runif(nz0,0.25,2),
+             "t" = runif(nz0,0.25,1),           
+             "scale1" = runif(qz,0.25,2),
+             "scale2" = runif(qz,0.25,2),
+             
+             "rho1" = rho1,
+             "rho2" = rho2,
+             "rho3" = rho3))}
+      inits.internal <- list(init( ));
+      if(n.chain >= 2) {
+        for(j in 2:n.chain) inits.internal <- c(inits.internal,list(init()))} 
+    } else{
+  init <- function(rngname, rngseed ){
     rho1 <- runif(1,-0.5,0.5)
     rho2 <- runif(1,-0.5,0.5) 
     rho3 <- runif(1, rho1*rho2 - sqrt((1-rho1^2)*(1-rho2^2)), 
@@ -61,15 +94,20 @@ function(y, n, q, xmu.1, p.xmu, xsum.1, p.xsum,
            
            "rho1" = rho1,
            "rho2" = rho2,
-           "rho3" = rho3))}    
-  
+           "rho3" = rho3,
+           
+           .RNG.name = rngname, 
+           .RNG.seed = rngseed))}    
+    
   # 1b, 2d, 
   # 3 SigmaVC (sigma.VC1 or t),SigmaUN (scale1 or scale2),
   # 4 rho1,2,3
-  inits.internal <- list(init( ));
+  set.seed(seed[1]); inits.internal <- list(init("base::Super-Duper", seed[1]));
   if(n.chain >= 2) {
-    for(j in 2:n.chain) inits.internal <- c(inits.internal,list(init( ))) }   
-  
+    for(j in 2:n.chain){ 
+      set.seed(seed[j]); 
+      inits.internal <- c(inits.internal,list(init("base::Wichmann-Hill",seed[j])))}}  
+    }
   if(!is.null(inits)){
     
   for(i in 1:n.chain){
